#include "sdkconfig.h"
#include "soc/rtc_cntl_reg.h"
#include "soc/rtc_io_reg.h"
#include "soc/soc_ulp.h"
#include "soc/sens_reg.h"

.macro push rx
	st \rx,r3,0
	sub r3,r3,1
.endm

.macro pop rx
	add r3,r3,1
	ld \rx,r3,0
.endm

.macro psr sr=r1 pos=.
	.set _next2,(\pos+16)
	move \sr,_next2
	push \sr
.endm

.macro ret sr=r1
	pop \sr
	jump \sr
.endm

.macro GPIO_IN
	psr
	jump set_in_one_wire_pin
.endm

.macro GPIO_OUT
	psr
	jump set_out_one_wire_pin
.endm

.macro GPIO_HIGH
	psr
	jump set_one_wire_pin
.endm

.macro GPIO_LOW
	psr
	jump clear_one_wire_pin
.endm

	.bss

	.global stack
stack:
	.skip 24

	.global stackEnd
stackEnd:
	.long 0

	.global dataR0
dataR0:
	.long 0

.data

   .global set_out_one_wire_pin
set_out_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_ENABLE_W1TS_REG, RTC_GPIO_ENABLE_W1TS_S + 0, 1, 1)
	ret

   .global set_in_one_wire_pin
set_in_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_ENABLE_W1TC_REG, RTC_GPIO_ENABLE_W1TC_S + 0, 1, 1)
	ret

   .global set_one_wire_pin
set_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_OUT_W1TS_REG, RTC_GPIO_ENABLE_W1TS_S + 0, 1, 1)
	ret

   .global clear_one_wire_pin
clear_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_OUT_W1TC_REG, RTC_GPIO_ENABLE_W1TC_S + 0, 1, 1)
	ret

	.text
	.global entry
entry:
	move r3, stackEnd

	psr
	jump get_temp

	jump exit_loop

rst_pulse:
	GPIO_OUT
	GPIO_LOW
	wait(4350) //500 us
	GPIO_HIGH
	GPIO_IN
	wait(240) //30us
	.global get_pin0
get_pin0:
	READ_RTC_REG(RTC_GPIO_IN_REG, RTC_GPIO_IN_NEXT_S + 0, 1) // r0 holds input result
	wait(3760) //470us
	ret

get_temp:
	psr
	jump rst_pulse
	//send 0xCC and 0x44 command
	move r2, 0xCC
	psr
	jump send_byte
	move r2, 0x44
	psr
	jump send_byte
	ret

send_byte:
	move r1, r2
	move r0, 0
send_byte_loop:
	rsh r2, r1, r0
	and r2, r2, 0x01
	push r0
	push r1
	psr
	jump send_bit
	pop r1
	pop r0
	add r0, r0, 1
	jumpr send_byte_loop, 8, LT
	wait(800) //100us
	ret

send_bit:
	GPIO_OUT
	GPIO_LOW
	wait(30) //5us
	sub r2, r2, 0
	jump send_bit_low, eq
send_bit_high:
	GPIO_HIGH
send_bit_low:
	wait(640) /* 80us */
	GPIO_HIGH
	ret

exit_loop:
	//delay(4 sec)
	move r0, 4000
	psr
	jump delay_ms

	jump wake_up
	/* End program */

	//delay
delay_ms:
	delay_ms_loop:
	sub r0, r0, 1
	wait(7990) /* 1millsecond = 1000 microsecond */
	jumpr delay_ms_loop, 1, GE
	ret
